import { FormControl, FormGroup } from '@angular/forms';
import { AsyncSubject, Observable, Subject, ReplaySubject, BehaviorSubject, concat, of } from 'rxjs';
import stringify from 'fast-json-stable-stringify';
import { __awaiter } from 'tslib';
import { filter, map, take, tap, debounceTime } from 'rxjs/operators';
import { ɵɵdefineInjectable, ɵsetClassMetadata, Injectable, ɵɵdefineDirective, ɵɵNgOnChangesFeature, ɵɵdefinePipe, Pipe, ɵɵdirectiveInject, ElementRef, ɵɵclassProp, Directive, HostBinding, Input, ɵɵdefineNgModule, ɵɵdefineInjector, ɵɵsetNgModuleScope, NgModule, InjectionToken, ɵɵinject, Inject, Optional } from '@angular/core';

function callFuncOnFormGroupAndChildren(formGroup, nameOfFunc) {
    if (!formGroup.controls)
        return;
    Object.keys(formGroup.controls)
        .map(formControlName => formGroup.controls[formControlName])
        .forEach(control => {
        if (control instanceof FormControl) {
            control[nameOfFunc]();
        }
        else if (control instanceof FormGroup) {
            this.callFuncOnFormGroupAndChildren(control);
        }
    });
}
/**
 * marks the form and all it's children, including other formGroups or controls
 * as touched
 * @param formGroup The form group to mark as touched
 */
function markFormGroupAsTouched(formGroup) {
    callFuncOnFormGroupAndChildren(formGroup, 'markAsTouched');
}
/**
 * marks the form and all it's children, including other formGroups or controls
 * as untouched
 * @param formGroup The form group to mark as untouched
 */
function markFormGroupAsUntouched(formGroup) {
    callFuncOnFormGroupAndChildren(formGroup, 'markAsUntouched');
}
/**
 * marks the form and all it's children, including other formGroups or controls
 * as dirty
 * @param formGroup The form group to mark as untouched
 */
function markFormGroupAsDirty(formGroup) {
    callFuncOnFormGroupAndChildren(formGroup, 'markAsDirty');
}
/**
 * marks the form and all it's children, including other formGroups or controls
 * as pristine
 * @param formGroup The form group to mark as pristine
 */
function markFormGroupAsPristine(formGroup) {
    callFuncOnFormGroupAndChildren(formGroup, 'markAsPristine');
}
/**
 * marks the form and all it's children, including other formGroups or controls
 * as pending
 * @param formGroup The form group to mark as pending
 */
function markFormGroupAsPending(formGroup) {
    callFuncOnFormGroupAndChildren(formGroup, 'markAsPending');
}
/**
 * get all dirty controls inside a form
 * @param form The form in which to check for dirty controls
 */
function getDirtyControls(form) {
    return Object.keys(form.controls)
        .map(k => ({ control: form.controls[k], name: k }))
        .filter(c => c.control.dirty);
}
class FormHelper {
}
FormHelper.markFormGroupAsTouched = markFormGroupAsTouched;
FormHelper.getDirtyControls = getDirtyControls;
FormHelper.markFormGroupAsUntouched = markFormGroupAsUntouched;
FormHelper.markFormGroupAsDirty = markFormGroupAsDirty;
FormHelper.markFormGroupAsPristine = markFormGroupAsPristine;
FormHelper.markFormGroupAsPending = markFormGroupAsPending;

/**
 * Escapes a html string to be safe, for example to show inside HTML
 * @param unsafe The HTML string which to escape
 */
function escapeXml(unsafeHTML) {
    return unsafeHTML.replace(/[<>&'"]/g, (c) => {
        switch (c) {
            case '<': return '&lt;';
            case '>': return '&gt;';
            case '&': return '&amp;';
            case '\'': return '&apos;';
            case '"': return '&quot;';
        }
    });
}
/**
 * Checks if a string is a number
 * @param ch The string which to check for number
 */
function isNumber(ch) {
    return !isNaN(parseInt(ch));
}
/**
 * Waits for a property on an object to be not undefined
 * And returns this property in an observable
 * @param obj The object which holds the property
 * @param (obj: TObject) => TProperty propertyAccessor the property accessor
 *   function
 * @param (prop: TProperty) => boolean the default checks if the property is
 *   not undefined to be valid and finish waiting. Here you can provide a
 *   custom function, e.g. also checking if it is not null
 * @param number retries How many 100 milliseconds it should wait for the
 *   property
 * @returns Observable<TProperty>
 */
function waitForProperty(obj, propertyAccessor, isPropertyValidFunc = prop => prop !== undefined, retries = 50) {
    let retry = 0;
    const returnSubject = new AsyncSubject();
    const interval = setInterval(() => {
        const property = propertyAccessor(obj);
        if (isPropertyValidFunc(property)) {
            clearInterval(interval);
            returnSubject.next(property);
            returnSubject.complete();
        }
        retry++;
        if (retry > retries) {
            clearInterval(interval);
            returnSubject.error('Property still undefined after ' + retries + ' retries');
            returnSubject.complete();
        }
    }, 100);
    return returnSubject.asObservable();
}
/**
 * Sets a css class on the document body if it does not already exists
 * @param cssClass The CSS class which to set on the document body
 */
function setBodyCssClass(cssClass) {
    if (document.body.className.indexOf(cssClass) === -1)
        document.body.className += ' ' + cssClass;
}
/**
 * Removes a css class from the document body
 * @param cssClass The CSS Class which to remove
 */
function removeBodyCssClass(cssClass) {
    document.body.className = document.body.className.replace(cssClass, '');
}
/**
 * Toggles a CSS class on the document body
 * @param cssClass The class which to toggle on the document body
 */
function toggleBodyCssClass(cssClass) {
    if (document.body.className.indexOf(cssClass) === -1)
        document.body.className += ' ' + cssClass;
    else
        document.body.className = document.body.className.replace(cssClass, '');
}
/**
 * Handy function if you want to check both null and undefined
 * @param value The value which to check for null or undefined
 */
function isNullOrUndefined(value) {
    return value === undefined || value === null;
}
/**
 * Checks if an object is empty, meaning it has to properties
 * @param obj The object which to check for empty
 */
function isObjectEmpty(obj) {
    if (obj === undefined || obj === null) {
        return true;
    }
    return Object.keys(obj).length === 0;
}
/**
 * Takes an object converts it to json and then URI-Encodes it to be used in a
 * URL
 * @param obj The object which to encode
 */
function encodeURIJson(obj) {
    const json = JSON.stringify(obj);
    return encodeURIComponent(json);
}
/**
 * Takes an URI value which is a json string and converts it to a javascript
 * object
 * @param encodedJson The URIEncoded JSON
 */
function decodeURIJson(encodedJson) {
    const json = decodeURIComponent(encodedJson);
    return JSON.parse(json);
}
/***
 * Clones an object via json stringify and json parse (so classes with
 * functions won't work here...) However plain objects and arrays are deep
 * cloned, this is why this method exists Also important: Because of deep
 * cloning objects in arrays are also cloned, so perfect for duplicating an
 * array without having the same reference inside Most useful also for
 * immutable Redux-State
 * @param obj The object to clone
*/
function jsonDeepClone(obj) {
    const json = JSON.stringify(obj);
    return JSON.parse(json);
}
/***
 * Compares 2 objects if they are equal by converting to json and comparing the
 * string So this method does not compare the reference but all the values
 * inside
 * @param obj1 First object to compare
 * @param obj2 Second object to compare
 */
function jsonIsEqual(obj1, obj2) {
    const json1 = stringify(obj1);
    const json2 = stringify(obj2);
    return json1 === json2;
}
/***
 * Used to preload image. Returns a stream containing the Html Image Element on
 * success or the error in case of an error
 * @param url The image src url to load
 */
function loadImage(url) {
    return new Observable(subscriber => {
        const image = new Image();
        image.onload = (ev) => {
            subscriber.next(image);
            subscriber.complete();
        };
        image.onerror = (ev) => {
            subscriber.next(ev.error);
            subscriber.complete();
        };
        image.src = url;
    });
}
/***
 * Javascript has no methode for rounding a number to certain amount of digits,
 * so here it is
 * @param num The number which to round
 * @param digits How many digits after the comma
 */
function roundNumber(num, digits = 2) {
    const digitOperator = '1' + '0'.repeat(digits);
    return Math.round(num * +digitOperator) / +digitOperator;
}
function createThumbnailFromBase64Image(base64Image, targetSize) {
    return new Observable(subscriber => {
        const img = new Image();
        img.onload = () => {
            const width = img.width;
            const height = img.height;
            const canvas = document.createElement('canvas');
            const ctx = canvas.getContext('2d');
            canvas.width = canvas.height = targetSize;
            ctx.drawImage(img, width > height ? (width - height) / 2 : 0, height > width ? (height - width) / 2 : 0, width > height ? height : width, width > height ? height : width, 0, 0, targetSize, targetSize);
            subscriber.next(canvas.toDataURL());
            subscriber.complete();
        };
        img.src = base64Image;
    });
}
function safeCall(obj, functionToCallGetterFuncOrName, defaultValue = undefined, ...params) {
    if (!obj) {
        return defaultValue;
    }
    const functionToCall = typeof functionToCallGetterFuncOrName === 'string' ?
        obj[functionToCallGetterFuncOrName] : functionToCallGetterFuncOrName(obj);
    if (!functionToCall) {
        return defaultValue;
    }
    return functionToCall.call(obj, ...params);
}
class MiscHelper {
}
MiscHelper.safeCall = safeCall;
MiscHelper.createThumbnailFromBase64Image = createThumbnailFromBase64Image;
MiscHelper.escapeXml = escapeXml;
MiscHelper.isNumber = isNumber;
MiscHelper.waitForProperty = waitForProperty;
MiscHelper.setBodyCssClass = setBodyCssClass;
MiscHelper.removeBodyCssClass = removeBodyCssClass;
MiscHelper.toggleBodyCssClass = toggleBodyCssClass;
MiscHelper.isNullOrUndefined = isNullOrUndefined;
MiscHelper.isObjectEmpty = isObjectEmpty;
MiscHelper.encodeURIJson = encodeURIJson;
MiscHelper.decodeURIJson = decodeURIJson;
MiscHelper.jsonDeepClone = jsonDeepClone;
MiscHelper.jsonIsEqual = jsonIsEqual;
MiscHelper.roundNumber = roundNumber;
MiscHelper.loadImage = loadImage;

/**
 * Creates a hidden link to download something given the href url
 * @param url The url which to download
 * @param fileName The name of the file to be downloaded (used for the a tag)
 */
function createAndClickDownloadLink(url, fileName) {
    const link = document.createElement('a');
    if (link.download === undefined)
        return;
    link.setAttribute('href', url);
    link.setAttribute('download', fileName);
    link.style.visibility = 'hidden';
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
}
/**
 * Creates a csv data uri from an array, this uri can then be downloaded in the browser
 * using createAndClickDownloadLink
 * @param data an array of string arrays containing the csv data
 */
function createCsvDataUriFromArray(data, delimiter = ';') {
    let csvContent = 'data:text/csv;charset=utf-8,';
    data.forEach(rowArray => {
        const row = rowArray.join(delimiter);
        csvContent += row + '\r\n';
    });
    return csvContent;
}
/**
 * Creates a csv file from the array using createCsvDataUriFromArray
 * And presents it to the user as a download file in the browser
 * @param data an array of string arrays containing the csv data
 * @param filename the name of the csv file which will be downloaded
 */
function downloadArrayAsCsvFile(data, filename, delimiter = ';') {
    const csv = createCsvDataUriFromArray(data, delimiter);
    createAndClickDownloadLink(csv, filename);
}
/**
 * opens a file dialog to choose a file and returns the file object
 * including the data.
 * When it is an image the data is read as base64, so the dataType is 'base64'
 * All other files are read as binaryString
 * @param fileDialogAccept this is optional, choose the mime types of files
 * the user can select in the file dialog
 */
function getFile(fileDialogAccept = '*') {
    return __awaiter(this, void 0, void 0, function* () {
        const p = new Promise((resolve, reject) => {
            const element = document.createElement('div');
            element.innerHTML = `<input type="file" accept="${fileDialogAccept}">`;
            const fileInput = element.firstChild;
            fileInput.addEventListener('change', () => {
                const file = fileInput.files[0];
                const reader = new FileReader();
                const doResolve = (data, dataType) => {
                    element.remove();
                    resolve({
                        size: file.size,
                        lastModified: file.lastModified,
                        mimeType: file.type,
                        name: file.name,
                        dataType: dataType,
                        data: data
                    });
                };
                if (file.type.startsWith('image')) {
                    reader.onload = () => {
                        const s = reader.result;
                        const base64Part = s.substring(s.indexOf('base64,') + 7);
                        doResolve(base64Part, 'base64');
                    };
                    reader.readAsDataURL(file);
                }
                else {
                    reader.onload = () => {
                        doResolve(reader.result, 'binaryString');
                    };
                    reader.readAsBinaryString(file);
                }
            });
            fileInput.click();
        });
        return p;
    });
}
class FileHelper {
}
FileHelper.getFile = getFile;
FileHelper.createAndClickDownloadLink = createAndClickDownloadLink;
FileHelper.createCsvDataUriFromArray = createCsvDataUriFromArray;
FileHelper.downloadArrayAsCsvFile = downloadArrayAsCsvFile;

/**
 * Padding the string left
 * @param s the string which to pad
 * @param length the length the string should be padded to
 * @param padChar the char which to use to fill up the string
 */
function padLeft(s, length, padChar) {
    while (s.length < length)
        s = padChar + s;
    return s;
}
/**
 * Padding the string right
 * @param s the string which to pad
 * @param length the length the string should be padded to
 * @param padChar the char which to use to fill up the string
 */
function padRight(s, length, padChar) {
    while (s.length < length)
        s += padChar;
    return s;
}
/**
 * Replace all occurences in the string
 * @param str The string in which to replace something
 * @param find The string to replace
 * @param replace The string that will replace the find string
 */
function replaceAll(str, find, replace) {
    let ret = '';
    for (const ch of str) {
        if (ch === find) {
            ret += replace;
        }
        else {
            ret += ch;
        }
    }
    return ret;
}
class StringHelper {
}
StringHelper.replaceAll = replaceAll;
StringHelper.padLeft = padLeft;
StringHelper.padRight = padRight;

function cSharpTicks2Date(ticks) {
    // ticks are in nanotime; convert to microtime
    const ticksToMicrotime = ticks / 10000;
    // ticks are recorded from 1/1/1; get microtime difference from 1/1/1/ to 1/1/1970
    const epochMicrotimeDiff = 62135600400000; // Math.abs(new Date(0, 0, 1, 0, 0, 0, 0).setFullYear(1));
    // new date is ticks, converted to microtime, minus difference from epoch microtime
    const tickDate = new Date(ticksToMicrotime - epochMicrotimeDiff);
    return tickDate;
}
function dateAddYears(date, years) {
    const result = new Date(date);
    result.setFullYear(result.getFullYear() + years);
    return result;
}
function dateSubstractYears(date, years) {
    const result = new Date(date);
    result.setFullYear(result.getFullYear() - years);
    return result;
}
function dateAddMonths(date, months) {
    const result = new Date(date);
    result.setMonth(result.getMonth() + months);
    return result;
}
function dateSubstractMonths(date, months) {
    const result = new Date(date);
    result.setMonth(result.getMonth() - months);
    return result;
}
function dateAddDays(date, days) {
    const result = new Date(date);
    result.setDate(result.getDate() + days);
    return result;
}
function dateSubstractDays(date, days) {
    const result = new Date(date);
    result.setDate(result.getDate() - days);
    return result;
}
function dateAddHours(date, hours) {
    const result = new Date(date);
    result.setHours(result.getHours() + hours);
    return result;
}
function dateSubstractHours(date, hours) {
    const result = new Date(date);
    result.setHours(result.getHours() - hours);
    return result;
}
function dateAddMinutes(date, minutes) {
    const result = new Date(date);
    result.setMinutes(result.getMinutes() + minutes);
    return result;
}
function dateSubstractMinutes(date, minutes) {
    const result = new Date(date);
    result.setMinutes(result.getMinutes() - minutes);
    return result;
}
function dateAddSeconds(date, seconds) {
    const result = new Date(date);
    result.setSeconds(result.getSeconds() + seconds);
    return result;
}
function dateSubstractSeconds(date, seconds) {
    const result = new Date(date);
    result.setSeconds(result.getSeconds() - seconds);
    return result;
}
function dateAddMilliSeconds(date, milliseconds) {
    const result = new Date(date);
    result.setMilliseconds(result.getMilliseconds() + milliseconds);
    return result;
}
function dateSubstractMilliSeconds(date, milliseconds) {
    const result = new Date(date);
    result.setMilliseconds(result.getMilliseconds() - milliseconds);
    return result;
}
function isToday(date) {
    let ret = true;
    const now = new Date(Date.now());
    if (now.getFullYear() !== date.getFullYear())
        ret = false;
    if (now.getMonth() !== date.getMonth())
        ret = false;
    if (now.getDate() !== date.getDate())
        ret = false;
    return ret;
}
function isYesterday(date) {
    let ret = true;
    const yesterday = dateSubstractDays(new Date(Date.now()), 1);
    if (yesterday.getFullYear() !== date.getFullYear())
        ret = false;
    if (yesterday.getMonth() !== date.getMonth())
        ret = false;
    if (yesterday.getDate() !== date.getDate())
        ret = false;
    return ret;
}
var IsNowGranulartiy;
(function (IsNowGranulartiy) {
    IsNowGranulartiy[IsNowGranulartiy["Hours"] = 0] = "Hours";
    IsNowGranulartiy[IsNowGranulartiy["Minutes"] = 1] = "Minutes";
    IsNowGranulartiy[IsNowGranulartiy["Seconds"] = 2] = "Seconds";
    IsNowGranulartiy[IsNowGranulartiy["Milliseconds"] = 3] = "Milliseconds";
})(IsNowGranulartiy || (IsNowGranulartiy = {}));
function isNow(date, granularity = IsNowGranulartiy.Minutes) {
    let ret = true;
    const now = new Date(Date.now());
    if (now.getFullYear() !== date.getFullYear())
        ret = false;
    if (now.getMonth() !== date.getMonth())
        ret = false;
    if (now.getDate() !== date.getDate())
        ret = false;
    if (now.getHours() !== date.getHours())
        ret = false;
    if (now.getMinutes() !== date.getMinutes() && granularity >= IsNowGranulartiy.Minutes)
        ret = false;
    if (now.getSeconds() !== date.getSeconds() && granularity >= IsNowGranulartiy.Seconds)
        ret = false;
    if (now.getMilliseconds() !== date.getMilliseconds() && granularity >= IsNowGranulartiy.Milliseconds)
        ret = false;
    return ret;
}
/***
 * Checks if the given object is a date object
 * @param obj the object to check for if it is a date or not
 */
function isDate(obj) {
    if (typeof obj !== 'object') {
        return false;
    }
    if (obj.constructor.name !== 'Date') {
        return false;
    }
    return true;
}
class DateHelper {
}
DateHelper.isDate = isDate;
DateHelper.isNow = isNow;
DateHelper.isToday = isToday;
DateHelper.isYesterday = isYesterday;
DateHelper.cSharpTicks2Date = cSharpTicks2Date;
DateHelper.dateAddYears = dateAddYears;
DateHelper.dateSubstractYears = dateSubstractYears;
DateHelper.dateAddMonths = dateAddMonths;
DateHelper.dateSubstractMonths = dateSubstractMonths;
DateHelper.dateAddDays = dateAddDays;
DateHelper.dateSubstractDays = dateSubstractDays;
DateHelper.dateAddHours = dateAddHours;
DateHelper.dateSubstractHours = dateSubstractHours;
DateHelper.dateAddMinutes = dateAddMinutes;
DateHelper.dateSubstractMinutes = dateSubstractMinutes;
DateHelper.dateAddSeconds = dateAddSeconds;
DateHelper.dateSubstractSeconds = dateSubstractSeconds;
DateHelper.dateAddMilliSeconds = dateAddMilliSeconds;
DateHelper.dateSubstractMilliSeconds = dateSubstractMilliSeconds;

/**
 * Merges 2 objects in regards that properties that are not found on the
 * mainObject are looked up on the extendedObject
 * @param mainObject Properties are first read here. if e.g. toString()
 * exists on both objects it will be returned from this one
 * @param extendedObject This is the second object that extends the mainObject.
 * Properties not found on the  mainObject are read from this one
 */
function createAutoIndexer(mainObject, extendedObject) {
    const handler = {
        get: (obj, prop) => {
            return prop in mainObject ?
                mainObject[prop] :
                extendedObject[prop];
        },
        set: (obj, prop, value) => {
            return prop in mainObject ?
                mainObject[prop] = value :
                extendedObject[prop] = value;
        }
    };
    return new Proxy(mainObject, handler);
}
/***
 * Checks if the given object has a specific constructor name
 * e.g: hasConstructorName(new Date(), 'Date')
 * @param obj the object to check for if it has the correct constructor name
 */
function hasConstructorName(obj, constructorName) {
    if (typeof obj !== 'object') {
        return false;
    }
    if (obj.constructor.name !== constructorName) {
        return false;
    }
    return true;
}
class ObjectHelper {
}
ObjectHelper.hasConstructorName = hasConstructorName;
ObjectHelper.createAutoIndexer = createAutoIndexer;

function move(arr, prevIndex, newIndex) {
    while (prevIndex < 0) {
        prevIndex += arr.length;
    }
    while (newIndex < 0) {
        newIndex += arr.length;
    }
    if (newIndex >= arr.length) {
        let k = newIndex - arr.length;
        while ((k--) + 1) {
            arr.push(undefined);
        }
    }
    arr.splice(newIndex, 0, arr.splice(prevIndex, 1)[0]);
    return arr;
}
class ArrayHelper {
}
ArrayHelper.move = move;

/***
 * Compares 2 objects if they are equal by converting to json and comparing the
 * string So this method does not compare the reference but all the values
 * inside
 * @param obj1 First object to compare
 * @param obj2 Second object to compare
 */
function objectIsEqual(obj1, obj2) {
    const json1 = stringify(obj1);
    const json2 = stringify(obj2);
    return json1 === json2;
}
/***
 * Compares 2 dates, dates can also be undefined or null
 * if both are null or undefined they are also the same...
 * @param date1
 * @param date2
 */
function isDateEqual(date1, date2) {
    if (!date1 && !date2) {
        return true;
    }
    if (!date1 || !date2) {
        return false;
    }
    if (date1 instanceof Date && date2 instanceof Date) {
        return date1.toISOString() === date2.toISOString();
    }
    throw new Error('isDateEqual: date1 or date2 must be unset or of type Date');
}
class ComparatorHelper {
}
ComparatorHelper.objectIsEqual = objectIsEqual;
ComparatorHelper.isDateEqual = isDateEqual;

class Helper {
}
Helper.date = DateHelper;
Helper.form = FormHelper;
Helper.misc = MiscHelper;
Helper.file = FileHelper;
Helper.string = StringHelper;
Helper.object = ObjectHelper;
Helper.array = ArrayHelper;
Helper.comparator = ComparatorHelper;

/**
 * Simple PubSub Service for communication between components and/or browser
 * tabs
 */
class PubSubService {
    constructor() {
        this.subject = new Subject();
        window.addEventListener('storage', (event) => {
            if (event.key !== PubSubService.LOCALSTORAGE_KEY)
                return;
            if (!event.newValue)
                return;
            const data = JSON.parse(event.newValue);
            this.publish(data.topic, data.message);
        });
    }
    /**
     * Publish across browser tabs
     * @param topic Name of the topic
     * @param message The message of the topic
     * @param includeSelf True if you want to publish also to the current
     * active tab (defaults to false)
     */
    publishCrossTab(topic, message, includeSelf = false) {
        localStorage.setItem(PubSubService.LOCALSTORAGE_KEY, JSON.stringify({ topic: topic, message: message }));
        if (includeSelf)
            this.publish(topic, message);
    }
    /**
     * Publish a topic inside the current browser tab
     * @param topic Name of the topic
     * @param message Message of the topic
     */
    publish(topic, message) {
        this.subject.next({ topic: topic, message: message });
    }
    /**
     * Subscribe to a topics and receive messages
     * @param topics One or more topics to listen to
     */
    topic(...topics) {
        return this.subject
            .asObservable().pipe(filter(data => topics.includes(data.topic)), map(data => data.message));
    }
}
PubSubService.LOCALSTORAGE_KEY = '__crossTabCommunication';
/** @nocollapse */ PubSubService.ɵfac = function PubSubService_Factory(t) { return new (t || PubSubService)(); };
/** @nocollapse */ PubSubService.ɵprov = ɵɵdefineInjectable({ token: PubSubService, factory: PubSubService.ɵfac, providedIn: 'root' });
/*@__PURE__*/ (function () { ɵsetClassMetadata(PubSubService, [{
        type: Injectable,
        args: [{
                providedIn: 'root'
            }]
    }], function () { return []; }, null); })();

class TemplateService {
    constructor() {
        this.templates = new Map();
    }
    get(templateName) {
        return this.templates.get(templateName);
    }
    has(templateName) {
        return this.templates.has(templateName);
    }
    put(templateName, templateRef) {
        this.templates.set(templateName, templateRef);
    }
}
/** @nocollapse */ TemplateService.ɵfac = function TemplateService_Factory(t) { return new (t || TemplateService)(); };
/** @nocollapse */ TemplateService.ɵprov = ɵɵdefineInjectable({ token: TemplateService, factory: TemplateService.ɵfac });
/*@__PURE__*/ (function () { ɵsetClassMetadata(TemplateService, [{
        type: Injectable
    }], function () { return []; }, null); })();

class CachedObservable {
    constructor(cacheDurationSeconds, observable) {
        this.cacheDurationSeconds = cacheDurationSeconds;
        this.observable = observable;
        this.replaySubject = new ReplaySubject(1);
        this.lastUpdatedSeconds = 0;
    }
    /***
     * Resets the cache duration, so the next attempt will fetch new
     */
    reset() {
        this.lastUpdatedSeconds = Date.now() / 1000 - this.cacheDurationSeconds - 1;
    }
    /***
     * Refreshes (refetches) the observable
     */
    refresh() {
        this.observable.pipe(take(1), tap(() => this.lastUpdatedSeconds =
            Date.now() / 1000), tap(response => this.replaySubject.next(response))).subscribe();
    }
    getObservable() {
        // Update the cache ?
        if (this.lastUpdatedSeconds <
            Date.now() / 1000 - this.cacheDurationSeconds) {
            return this.observable.pipe(take(1), tap(() => this.lastUpdatedSeconds =
                Date.now() / 1000), tap(response => {
                this.replaySubject.next(response);
            }));
        }
        return this.replaySubject.asObservable();
    }
}

class CachedObservableService {
    constructor() {
        this.dictionary = new Map();
    }
    resetAll() {
        this.dictionary.forEach((observable, key) => {
            observable.reset();
        });
    }
    resetByKeyStartsWith(keyStartsWith) {
        this.dictionary.forEach((observable, key) => {
            if (key.startsWith(keyStartsWith)) {
                observable.reset();
            }
        });
    }
    /***
     * Resets the cache duration, so the next attempt will fetch new
     */
    reset(key) {
        if (this.dictionary.has(key)) {
            this.dictionary.get(key).reset();
        }
    }
    /***
     * Refreshes (refetches) the observable
     */
    refresh(key) {
        if (this.dictionary.has(key)) {
            this.dictionary.get(key).refresh();
        }
    }
    getOrCreate(key, cacheDurationSeconds, observable) {
        if (!this.dictionary.has(key)) {
            this.dictionary.set(key, new CachedObservable(cacheDurationSeconds, observable));
        }
        return this.dictionary.get(key).getObservable();
    }
}
/** @nocollapse */ CachedObservableService.ɵfac = function CachedObservableService_Factory(t) { return new (t || CachedObservableService)(); };
/** @nocollapse */ CachedObservableService.ɵprov = ɵɵdefineInjectable({ token: CachedObservableService, factory: CachedObservableService.ɵfac, providedIn: 'root' });
/*@__PURE__*/ (function () { ɵsetClassMetadata(CachedObservableService, [{
        type: Injectable,
        args: [{
                providedIn: 'root'
            }]
    }], function () { return []; }, null); })();

class Services {
}
Services.PubSubService = PubSubService;
Services.TemplateService = TemplateService;
Services.CachedObservableService = CachedObservableService;

/**
 * This should be the very base component of every angular component
 * It uses OnInit and OnDestroy, so remember to call super.ngOnInit() at first
 * if you also use OnInit in the child class
 *
 * In nearly every angular component you will subscribe to something
 * and the best approach to unsubscribe is: takeUntil
 * takeUntil the component gets destroyed
 *
 * e.g.:
 * timer(1000).pipe(
 *  takeUntil(this.onDestroy$),
 *  tap(() => do something)
 * ).subscribe();
 *
 * so this base class has no injector dependencies and provide the onDestroy$
 * and onInit$ observables which you can subscribe to.
 *
 * e.g. you don't have to use OnInit in the subsclass you cann also just
 * subscribe to this.onInit$.subscribe() in the constructor
 * it' also useful if you want to access @Input variables which get set
 * at in onInit lifecycle and not in the constructor, so:
 *
 * @Input() someVar;
 * constructor() {
 *   this.onInit$().subscribe(() => console.log(this.someVar));
 * }
 */
class BaseComponent {
    constructor() {
        this.isDisabled$ = new BehaviorSubject(false);
        this.onDestroy$ubject = new ReplaySubject(1);
        this.onInit$ubject = new ReplaySubject(1);
        this.onChanges$ubject = new Subject();
        this.onDestroy$ = this.onDestroy$ubject.asObservable();
        this.onInit$ = this.onInit$ubject.asObservable();
        this.onChanges$ = this.onChanges$ubject.asObservable();
    }
    set disabled(isDisabled) {
        this.isDisabled$.next(isDisabled);
    }
    /**
     * Returns the SimpleChange event for the given property
     * @param property The property (name of the @Input) for which to filter
     */
    onChangesByProperty$(property) {
        return this.onChanges$.pipe(filter(ch => ch.property === property), map(ch => ch.change));
    }
    /**
     * Returns the currentValue of the SimpleChange event for the given property
     * @param property The property (name of the @Input) for which to filter
     */
    onChangesCurrentValueByProperty$(property) {
        return this.onChanges$.pipe(filter(ch => ch.property === property), map(ch => ch.change.currentValue));
    }
    ngOnInit() {
        this.onInit$ubject.next(undefined);
        this.onInit$ubject.complete();
    }
    ngOnDestroy() {
        this.onDestroy$ubject.next(undefined);
        this.onDestroy$ubject.complete();
        this.onChanges$ubject.complete();
    }
    ngOnChanges(changes) {
        const changedProperties = Object.keys(changes);
        changedProperties.forEach(changedProperty => {
            this.onChanges$ubject.next({
                property: changedProperty,
                change: changes[changedProperty]
            });
        });
    }
}
/** @nocollapse */ BaseComponent.ɵfac = function BaseComponent_Factory(t) { return new (t || BaseComponent)(); };
/** @nocollapse */ BaseComponent.ɵdir = ɵɵdefineDirective({ type: BaseComponent, inputs: { disabled: "disabled" }, features: [ɵɵNgOnChangesFeature] });

/***
 * Checks if a form control date is greater than another form control date
 * @param lowerDateControl the other control which holds the date that must be lower
 * @param formValue2DateFunc if the value of the form controls is not a date provide
 * this function to convert to date
 */
function dateGreaterThan(lowerDateControl, formValue2DateFunc) {
    return (control) => {
        let lowerDate;
        let greaterDate;
        if (formValue2DateFunc) {
            lowerDate = formValue2DateFunc(lowerDateControl.value);
            greaterDate = formValue2DateFunc(control.value);
        }
        else {
            lowerDate = lowerDateControl.value;
            greaterDate = control.value;
        }
        return lowerDate > greaterDate ?
            { dateGreaterThan: { value: control.value } } : null;
    };
}
/***
 * This validator function must  be used on the formGroup level, see
 * https://angular.io/guide/form-validation#cross-field-validation
 * @param controlNames the controlNames which at least on must  have a value
 * @param hasValueFunc Optional a function to tell if the control has a value or not
 */
function anyRequired(controlNames, hasValueFunc = c => !!c.value) {
    return (fb) => {
        if (!hasConstructorName(fb, 'FormGroup')) {
            throw new Error('Validator anyRequired must be on form level. See https://angular.io/guide/form-validation#cross-field-validation');
        }
        const someHasValue = controlNames.some(controlName => hasValueFunc(fb.get(controlName)));
        return someHasValue ? null : { anyRequired: true };
    };
}
class FormValidators {
}
FormValidators.anyRequired = anyRequired;
FormValidators.dateGreaterThan = dateGreaterThan;

class RoundNumberPipe {
    transform(value, digits) {
        return roundNumber(value, digits);
    }
}
/** @nocollapse */ RoundNumberPipe.ɵfac = function RoundNumberPipe_Factory(t) { return new (t || RoundNumberPipe)(); };
/** @nocollapse */ RoundNumberPipe.ɵpipe = ɵɵdefinePipe({ name: "roundNumber", type: RoundNumberPipe, pure: true });
/*@__PURE__*/ (function () { ɵsetClassMetadata(RoundNumberPipe, [{
        type: Pipe,
        args: [{ name: 'roundNumber' }]
    }], null, null); })();

class NgIonicSyncFormcontrolClassesDirective {
    constructor(el) {
        this.el = el;
        this.ionDirty = false;
        this.ionPristine = true;
        this.ionTouched = false;
        this.ionUnTouched = true;
        const setIonItemClass = (cssClass, visible) => {
            if (!this.ionItem) {
                return;
            }
            if (!visible) {
                this.ionItem.classList.remove(cssClass);
            }
            else if (!this.ionItem.className.includes(cssClass)) {
                this.ionItem.classList.add(cssClass);
            }
        };
        const checkMutations = new Subject();
        checkMutations.pipe(debounceTime(100), tap(() => {
            const cssClasses = el.nativeElement.className;
            if (cssClasses.includes('ng-touched')) {
                this.ionTouched = true;
                this.ionUnTouched = false;
                // If we have an ion-item as a parent also sync its classes
                setIonItemClass('ion-touched', true);
                setIonItemClass('ion-untouched', false);
            }
            if (cssClasses.includes('ng-untouched')) {
                this.ionUnTouched = true;
                this.ionTouched = false;
                // If we have an ion-item as a parent also sync its classes
                setIonItemClass('ion-touched', false);
                setIonItemClass('ion-untouched', true);
            }
            if (cssClasses.includes('ng-pristine')) {
                this.ionPristine = true;
                this.ionDirty = false;
                // If we have an ion-item as a parent also sync its classes
                setIonItemClass('ion-pristine', true);
                setIonItemClass('ion-dirty', false);
            }
            if (cssClasses.includes('ng-dirty')) {
                this.ionPristine = false;
                this.ionDirty = true;
                // If we have an ion-item as a parent also sync its classes
                setIonItemClass('ion-pristine', false);
                setIonItemClass('ion-dirty', true);
            }
        })).subscribe();
        this.mutationObserver = new MutationObserver(mutations => {
            // Use a subject for debouncing
            checkMutations.next(mutations);
        });
        this.mutationObserver.observe(el.nativeElement, {
            attributes: true,
            attributeFilter: ['style', 'class']
        });
    }
    get ionItem() {
        if (!this._ionItem) {
            this._ionItem = this.el.nativeElement.closest('ion-item');
        }
        return this._ionItem;
    }
    ngOnDestroy() {
        this.mutationObserver.disconnect();
    }
}
/** @nocollapse */ NgIonicSyncFormcontrolClassesDirective.ɵfac = function NgIonicSyncFormcontrolClassesDirective_Factory(t) { return new (t || NgIonicSyncFormcontrolClassesDirective)(ɵɵdirectiveInject(ElementRef)); };
/** @nocollapse */ NgIonicSyncFormcontrolClassesDirective.ɵdir = ɵɵdefineDirective({ type: NgIonicSyncFormcontrolClassesDirective, selectors: [["", "ngIonSync", ""]], hostVars: 8, hostBindings: function NgIonicSyncFormcontrolClassesDirective_HostBindings(rf, ctx) { if (rf & 2) {
        ɵɵclassProp("ion-dirty", ctx.ionDirty)("ion-pristine", ctx.ionPristine)("ion-touched", ctx.ionTouched)("ion-untouched", ctx.ionUnTouched);
    } } });
/*@__PURE__*/ (function () { ɵsetClassMetadata(NgIonicSyncFormcontrolClassesDirective, [{
        type: Directive,
        args: [{
                // tslint:disable-next-line:directive-selector
                selector: '[ngIonSync]'
            }]
    }], function () { return [{ type: ElementRef }]; }, { ionDirty: [{
            type: HostBinding,
            args: ['class.ion-dirty']
        }], ionPristine: [{
            type: HostBinding,
            args: ['class.ion-pristine']
        }], ionTouched: [{
            type: HostBinding,
            args: ['class.ion-touched']
        }], ionUnTouched: [{
            type: HostBinding,
            args: ['class.ion-untouched']
        }] }); })();

class ShadowDomStyleDirective {
    constructor(el) {
        this.el = el;
    }
    set shadowStyle(value) {
        const elem = this.el.nativeElement;
        waitForProperty(elem, obj => elem.shadowRoot.querySelector(value.selector))
            .subscribe((shadowElem) => {
            if (!shadowElem || !value) {
                return;
            }
            for (const style of Object.keys(value.style)) {
                if (!shadowElem.style || !value.style) {
                    continue;
                }
                shadowElem.style[style] = value.style[style];
            }
        });
    }
}
/** @nocollapse */ ShadowDomStyleDirective.ɵfac = function ShadowDomStyleDirective_Factory(t) { return new (t || ShadowDomStyleDirective)(ɵɵdirectiveInject(ElementRef)); };
/** @nocollapse */ ShadowDomStyleDirective.ɵdir = ɵɵdefineDirective({ type: ShadowDomStyleDirective, selectors: [["", "shadowStyle", ""]], inputs: { shadowStyle: "shadowStyle" } });
/*@__PURE__*/ (function () { ɵsetClassMetadata(ShadowDomStyleDirective, [{
        type: Directive,
        args: [{
                selector: '[shadowStyle]'
            }]
    }], function () { return [{ type: ElementRef }]; }, { shadowStyle: [{
            type: Input
        }] }); })();

class SwissArmyKnifeModule {
}
/** @nocollapse */ SwissArmyKnifeModule.ɵmod = ɵɵdefineNgModule({ type: SwissArmyKnifeModule });
/** @nocollapse */ SwissArmyKnifeModule.ɵinj = ɵɵdefineInjector({ factory: function SwissArmyKnifeModule_Factory(t) { return new (t || SwissArmyKnifeModule)(); }, imports: [[]] });
(function () { (typeof ngJitMode === "undefined" || ngJitMode) && ɵɵsetNgModuleScope(SwissArmyKnifeModule, { declarations: [RoundNumberPipe, NgIonicSyncFormcontrolClassesDirective, ShadowDomStyleDirective], exports: [RoundNumberPipe, NgIonicSyncFormcontrolClassesDirective, ShadowDomStyleDirective] }); })();
/*@__PURE__*/ (function () { ɵsetClassMetadata(SwissArmyKnifeModule, [{
        type: NgModule,
        args: [{
                declarations: [RoundNumberPipe, NgIonicSyncFormcontrolClassesDirective, ShadowDomStyleDirective],
                imports: [],
                exports: [RoundNumberPipe, NgIonicSyncFormcontrolClassesDirective, ShadowDomStyleDirective]
            }]
    }], null, null); })();

const KeyValueStoreInitialValue = new InjectionToken('KeyValueStoreInitialValue');
class KeyValueStoreService {
    constructor(initialStore) {
        this.initialStore = initialStore;
        this.LOCALSTORAGE_KEY = 'KeyValueStore';
        this.changes$ = new Subject();
        const currentValue = this.getFromLocalStorage(this.LOCALSTORAGE_KEY);
        this.store$ = new BehaviorSubject((currentValue || initialStore || {}));
        this.changes$.subscribe(change => {
            const store = this.store$.getValue();
            store[change.key] = change.currentValue;
            this.store$.next(store);
        });
        this.store$.subscribe(store => {
            localStorage.setItem(this.LOCALSTORAGE_KEY, JSON.stringify(store));
        });
    }
    getStore() {
        return this.store$.getValue();
    }
    getStore$() {
        return this.store$.asObservable();
    }
    getAllChanges$() {
        return this.changes$.asObservable();
    }
    resetStore() {
        this.store$ = new BehaviorSubject((this.initialStore || {}));
    }
    getValue$(key, startWithCurrentValue = true) {
        return startWithCurrentValue ?
            concat(of(this.getValue(key)), this.changes$.asObservable().pipe(filter(change => change.key === key), map(change => change.currentValue)))
            : this.changes$.asObservable().pipe(filter(change => change.key === key), map(change => change.currentValue));
    }
    getValueChanges$(key) {
        return this.changes$.asObservable().pipe(filter(change => change.key === key));
    }
    getFromLocalStorage(key) {
        try {
            return JSON.parse(localStorage.getItem(key));
        }
        catch (_a) {
            return undefined;
        }
    }
    getValue(key) {
        return this.getStore()[key];
    }
    setValue(key, value) {
        this.changes$.next({
            key: key,
            currentValue: value,
            previousValue: this.getFromLocalStorage(key)
        });
    }
}
/** @nocollapse */ KeyValueStoreService.ɵfac = function KeyValueStoreService_Factory(t) { return new (t || KeyValueStoreService)(ɵɵinject(KeyValueStoreInitialValue, 8)); };
/** @nocollapse */ KeyValueStoreService.ɵprov = ɵɵdefineInjectable({ token: KeyValueStoreService, factory: KeyValueStoreService.ɵfac, providedIn: 'root' });
/*@__PURE__*/ (function () { ɵsetClassMetadata(KeyValueStoreService, [{
        type: Injectable,
        args: [{
                providedIn: 'root'
            }]
    }], function () { return [{ type: undefined, decorators: [{
                type: Inject,
                args: [KeyValueStoreInitialValue]
            }, {
                type: Optional
            }] }]; }, null); })();

class StatefulObject {
    constructor() {
        this.stateSubject = new BehaviorSubject({});
        this.state$ = this.stateSubject.asObservable();
        this.stateSubject.subscribe(s => this.stateSnapshot = s);
    }
    setState(newState) {
        this.stateSubject.next(Object.assign(Object.assign({}, this.stateSubject.getValue()), newState));
    }
}

/*
 * Public API Surface of ng-swiss-army-knife
 */
class SwissArmyKnife {
}
SwissArmyKnife.FormValidators = FormValidators;
SwissArmyKnife.Helper = Helper;
SwissArmyKnife.Services = Services;
SwissArmyKnife.BaseComponent = BaseComponent;

/**
 * Generated bundle index. Do not edit.
 */

export { ArrayHelper, BaseComponent, CachedObservable, CachedObservableService, ComparatorHelper, DateHelper, FileHelper, FormHelper, FormValidators, Helper, IsNowGranulartiy, KeyValueStoreInitialValue, KeyValueStoreService, MiscHelper, NgIonicSyncFormcontrolClassesDirective, ObjectHelper, PubSubService, RoundNumberPipe, ShadowDomStyleDirective, StatefulObject, StringHelper, SwissArmyKnife, SwissArmyKnifeModule, TemplateService, anyRequired, cSharpTicks2Date, createAndClickDownloadLink, createAutoIndexer, createCsvDataUriFromArray, createThumbnailFromBase64Image, dateAddDays, dateAddHours, dateAddMilliSeconds, dateAddMinutes, dateAddMonths, dateAddSeconds, dateAddYears, dateGreaterThan, dateSubstractDays, dateSubstractHours, dateSubstractMilliSeconds, dateSubstractMinutes, dateSubstractMonths, dateSubstractSeconds, dateSubstractYears, decodeURIJson, downloadArrayAsCsvFile, encodeURIJson, escapeXml, getDirtyControls, getFile, hasConstructorName, isDate, isDateEqual, isNow, isNullOrUndefined, isNumber, isObjectEmpty, isToday, isYesterday, jsonDeepClone, jsonIsEqual, loadImage, markFormGroupAsDirty, markFormGroupAsPending, markFormGroupAsPristine, markFormGroupAsTouched, markFormGroupAsUntouched, move, objectIsEqual, padLeft, padRight, removeBodyCssClass, replaceAll, roundNumber, safeCall, setBodyCssClass, toggleBodyCssClass, waitForProperty };
//# sourceMappingURL=ng-swiss-army-knife.js.map
