import { Observable } from 'rxjs';
/**
 * Escapes a html string to be safe, for example to show inside HTML
 * @param unsafe The HTML string which to escape
 */
export declare function escapeXml(unsafeHTML: string): string;
/**
 * Checks if a string is a number
 * @param ch The string which to check for number
 */
export declare function isNumber(ch: string): boolean;
/**
 * Waits for a property on an object to be not undefined
 * And returns this property in an observable
 * @param obj The object which holds the property
 * @param (obj: TObject) => TProperty propertyAccessor the property accessor
 *   function
 * @param (prop: TProperty) => boolean the default checks if the property is
 *   not undefined to be valid and finish waiting. Here you can provide a
 *   custom function, e.g. also checking if it is not null
 * @param number retries How many 100 milliseconds it should wait for the
 *   property
 * @returns Observable<TProperty>
 */
export declare function waitForProperty<TObject, TProperty>(obj: TObject, propertyAccessor: (obj: TObject) => TProperty, isPropertyValidFunc?: (prop: TProperty) => boolean, retries?: number): Observable<TProperty>;
/**
 * Sets a css class on the document body if it does not already exists
 * @param cssClass The CSS class which to set on the document body
 */
export declare function setBodyCssClass(cssClass: string): void;
/**
 * Removes a css class from the document body
 * @param cssClass The CSS Class which to remove
 */
export declare function removeBodyCssClass(cssClass: string): void;
/**
 * Toggles a CSS class on the document body
 * @param cssClass The class which to toggle on the document body
 */
export declare function toggleBodyCssClass(cssClass: string): void;
/**
 * Handy function if you want to check both null and undefined
 * @param value The value which to check for null or undefined
 */
export declare function isNullOrUndefined(value: any): boolean;
/**
 * Checks if an object is empty, meaning it has to properties
 * @param obj The object which to check for empty
 */
export declare function isObjectEmpty(obj: any): boolean;
/**
 * Takes an object converts it to json and then URI-Encodes it to be used in a
 * URL
 * @param obj The object which to encode
 */
export declare function encodeURIJson(obj: any): string;
/**
 * Takes an URI value which is a json string and converts it to a javascript
 * object
 * @param encodedJson The URIEncoded JSON
 */
export declare function decodeURIJson<T>(encodedJson: string): T;
/***
 * Clones an object via json stringify and json parse (so classes with
 * functions won't work here...) However plain objects and arrays are deep
 * cloned, this is why this method exists Also important: Because of deep
 * cloning objects in arrays are also cloned, so perfect for duplicating an
 * array without having the same reference inside Most useful also for
 * immutable Redux-State
 * @param obj The object to clone
*/
export declare function jsonDeepClone<T>(obj: T): T;
/***
 * Compares 2 objects if they are equal by converting to json and comparing the
 * string So this method does not compare the reference but all the values
 * inside
 * @param obj1 First object to compare
 * @param obj2 Second object to compare
 */
export declare function jsonIsEqual(obj1: any, obj2: any): boolean;
/***
 * Used to preload image. Returns a stream containing the Html Image Element on
 * success or the error in case of an error
 * @param url The image src url to load
 */
export declare function loadImage(url: string): Observable<HTMLImageElement>;
/***
 * Javascript has no methode for rounding a number to certain amount of digits,
 * so here it is
 * @param num The number which to round
 * @param digits How many digits after the comma
 */
export declare function roundNumber(num: number, digits?: number): number;
export declare function createThumbnailFromBase64Image(base64Image: string, targetSize: number): Observable<string>;
/**
 * Safe calls a function, meaning if the function does not exist on the object
 * or the object is null/undefined it does not throw an error, but return
 * the defaultValue
 * @param obj The object which holds the function to call
 * @param functionToCallGetterFuncOrName a getter function to return the
 * functionToCall or the name of the function as a string
 * @param defaultValue an optional default value to return if the function
 * cannot be called
 * @param params optional parameters for the function
 */
declare type functionGetter<TObject, TReturnValue> = (obj: TObject) => (...params: any[]) => TReturnValue;
export declare function safeCall<TObject, TReturnValue>(obj: TObject, functionToCallGetterFuncOrName: string | functionGetter<TObject, TReturnValue>, defaultValue?: TReturnValue, ...params: any[]): TReturnValue;
export declare class MiscHelper {
    static safeCall: typeof safeCall;
    static createThumbnailFromBase64Image: typeof createThumbnailFromBase64Image;
    static escapeXml: typeof escapeXml;
    static isNumber: typeof isNumber;
    static waitForProperty: typeof waitForProperty;
    static setBodyCssClass: typeof setBodyCssClass;
    static removeBodyCssClass: typeof removeBodyCssClass;
    static toggleBodyCssClass: typeof toggleBodyCssClass;
    static isNullOrUndefined: typeof isNullOrUndefined;
    static isObjectEmpty: typeof isObjectEmpty;
    static encodeURIJson: typeof encodeURIJson;
    static decodeURIJson: typeof decodeURIJson;
    static jsonDeepClone: typeof jsonDeepClone;
    static jsonIsEqual: typeof jsonIsEqual;
    static roundNumber: typeof roundNumber;
    static loadImage: typeof loadImage;
}
export {};
